# pylint: disable=C0103,C0301,C0116

""" module docstring """

import logging
import os
import pathlib
import subprocess
import sys

# pylint: disable=W0611
from gqlib.profilers import RegionQuantifier
from gqlib.db.db_import import DomainBedDatabaseImporter

from . import __version__
from .handle_args import handle_args


logger = logging.getLogger(__name__)


def check_bwa_index(prefix):
    """ docstring """
    suffixes = (".amb", ".ann", ".bwt", ".pac", ".sa")
    return all(os.path.isfile(prefix + suffix) for suffix in suffixes)


# pylint: disable=R0913
def run_alignment(
    profiler,
    input_files,
    bwa_index,
    cpus_for_alignment=1,
    min_identity=None,
    min_seqlen=None,
    unmarked_orphans=False,
):
    """ docstring """
    commands = [
        f"bwa mem -v 1 -a -t {cpus_for_alignment} "
        f"-K 10000000 {bwa_index} {' '.join(input_files)}",
    ]

    commands = " | ".join(commands)

    logger.info("Used command: %s", commands)

    try:
        with subprocess.Popen(
            commands, shell=True, stdout=subprocess.PIPE
        ) as read_processing_proc:
            profiler.count_alignments(
                read_processing_proc.stdout,
                aln_format="sam",
                min_identity=min_identity,
                min_seqlen=min_seqlen,
                unmarked_orphans=unmarked_orphans,
            )
    except Exception as err:
        if isinstance(err, ValueError) and str(err).strip() == "file does not contain alignment data":
            logger.error("Failed to align. Is `bwa mem` installed?")
            sys.exit(1)
        logger.error("Caught some exception:")
        logger.error("%s", err)
        raise Exception from err


def check_input_reads(fwd=None, rev=None, singles=None, orphans=None):
    """ docstring """
    fwd_reads = fwd.split(",") if fwd else None
    rev_reads = rev.split(",") if rev else None
    single_reads = singles.split(",") if singles else None
    orphan_reads = orphans.split(",") if orphans else None

    all_readsets = []

    if fwd_reads and rev_reads:
        if len(fwd_reads) == len(rev_reads):
            all_readsets += zip(
                (["paired"] * len(fwd_reads)),
                fwd_reads, rev_reads
            )
        else:
            raise ValueError(
                f"Found different numbers of forward/R1 {len(fwd_reads)} "
                f"and reverse/R2 {len(rev_reads)} reads."
            )
    elif fwd_reads:
        logger.warning(
            "Found -1 forward/R1 reads but no -2 reverse/R2 reads. "
            "Treating these as single-end reads."
        )
        all_readsets += zip((["single"] * len(fwd_reads)), fwd_reads)
    elif rev_reads:
        raise ValueError(
            "Found -2 reverse/R2 reads but no -1 forward/R1 reads."
        )

    if single_reads:
        all_readsets += zip((["single"] * len(single_reads)), single_reads)
    if orphan_reads:
        all_readsets += zip((["orphan"] * len(orphan_reads)), orphan_reads)

    if not all_readsets:
        raise ValueError("No input reads specified.")

    for _, *reads in all_readsets:
        for r in reads:
            if not os.path.isfile(r):
                raise ValueError(f"{r} does not seem to be a valid read file.")

    return all_readsets


def main():

    args = handle_args(sys.argv[1:])

    logger.info("Version: %s", __version__)
    logger.info(
        "Command: %s %s",
        os.path.basename(sys.argv[0]), " ".join(sys.argv[1:])
    )

    print(args)

    input_data = check_input_reads(
        args.reads1, args.reads2,
        args.singles, args.orphans,
    )

    if not os.path.exists(args.annotation_db):
        raise ValueError(
            f"{args.annotation_db} is not a valid annotation database"
        )
    if not check_bwa_index(args.bwa_index):
        raise ValueError(f"{args.bwa_index} is not a valid bwa index.")

    if os.path.dirname(args.out_prefix):
        pathlib.Path(os.path.dirname(args.out_prefix)).mkdir(
            exist_ok=True, parents=True
        )

    db_importer = DomainBedDatabaseImporter(
        logger, args.annotation_db, single_category="cazy"
    )
    logger.info("Finished loading database.")

    profiler = RegionQuantifier(
        db=db_importer,
        out_prefix=args.out_prefix,
        ambig_mode="1overN",
        reference_type="domain",
    )

    for input_type, *reads in input_data:

        logger.info("Running %s alignment: %s", input_type, ",".join(reads))

        run_alignment(
            profiler,
            reads,
            args.bwa_index,
            cpus_for_alignment=args.cpus_for_alignment,
            min_identity=args.min_identity,
            min_seqlen=args.min_seqlen,
            unmarked_orphans=input_type == "orphan",
        )

    profiler.finalise(restrict_reports=("raw", "rpkm",))


if __name__ == "__main__":
    main()



# pylint: disable=C0301,C0103
""" module docstring """

import argparse
import logging
import textwrap

from . import __version__
from . import __toolname__


def handle_args(args):
    """ docstring """

    log_ap = argparse.ArgumentParser(prog=__toolname__, add_help=False)
    log_ap.add_argument(
        "-l", "--log_level",
        type=int, choices=range(1, 5), default=logging.INFO
    )
    log_args, _ = log_ap.parse_known_args(args)

    try:
        logging.basicConfig(
            level=log_args.log_level,
            format='[%(asctime)s] %(message)s'
        )
    except ValueError as invalid_loglevel_err:
        raise ValueError(
            f"Invalid log level: {log_args.log_level}"
        ) from invalid_loglevel_err

    ap = argparse.ArgumentParser(
        prog=__toolname__,
        formatter_class=argparse.RawTextHelpFormatter,
        parents=(log_ap,),
    )
    ap.add_argument(
        "annotation_db",
        type=str,
        help=textwrap.dedent(
            """\
            Path to a bed4 file containing the domain annotation.
            """
        ),
    )
    ap.add_argument(
        "bwa_index",
        type=str,
        help=textwrap.dedent(
            """\
            Path to the bwa reference index.
            """
        ),
    )

    ap.add_argument(
        "-1",
        dest="reads1",
        type=str,
        help="A comma-delimited string of forward/R1 read fastq files."
    )

    ap.add_argument(
        "-2",
        dest="reads2",
        type=str,
        help="A comma-delimited string of reverse/R2 read fastq files."
    )

    ap.add_argument(
        "--singles", "-s",
        type=str,
        help="A comma-delimited string of single-end read fastq files."
    )

    ap.add_argument(
        "--orphans",
        type=str,
        help="A comma-delimited string of orphan read fastq files."
    )

    ap.add_argument(
        "--out_prefix",
        "-o",
        type=str,
        default=__toolname__,
        help="Prefix for output files.",
    )

    ap.add_argument(
        "--min_identity",
        type=float,
        default=0.97,
        help="Minimum sequence identity [n_match/length] "
             "for an alignment to be considered.",
    )

    ap.add_argument(
        "--min_seqlen",
        type=int,
        default=45,
        help="Minimum read length [bp] for an alignment to be considered.",
    )

    ap.add_argument(
        "--cpus_for_alignment", "-t",
        type=int, default=1,
        help="",
    )

    ap.add_argument(
        "--version", "-v", action="version", version="%(prog)s " + __version__
    )
    ap.add_argument("--debug", action="store_true")

    return ap.parse_args(args)



# coding: utf-8
from setuptools import setup, find_packages
from setuptools.extension import Extension
from distutils.extension import Extension
from codecs import open
from os import path
import glob
import re
import sys

from cayman import __version__
here = path.abspath(path.dirname("__file__"))

with open(path.join(here, "DESCRIPTION.md"), encoding="utf-8") as description:
	description = long_description = description.read()

	name="cayman"
	version = __version__

	if sys.version_info.major != 3:
		raise EnvironmentError(f"""{name} is a python module that requires python3, and is not compatible with python2.""")

	setup(
		name=name,
		version=version,
		description=description,
		long_description=long_description,
		url="https://github.com/cschu/cayman",
		author="Christian Schudoma",
		author_email="cschu1981@gmail.com",
		license="MIT",
		classifiers=[
			"Development Status :: 4 - Beta",
			"Topic :: Scientific/Engineering :: Bio-Informatics",
			"License :: OSI Approved :: MIT License",
			"Operating System :: POSIX :: Linux",
			"Programming Language :: Python :: 3.7",
			"Programming Language :: Python :: 3.8",
			"Programming Language :: Python :: 3.9",
			"Programming Language :: Python :: 3.10",
		],
		zip_safe=False,
		keywords="",
		packages=find_packages(exclude=["test"]),
		install_requires=[line.strip() for line in open("requirements.txt", "rt")],
		entry_points={
			"console_scripts": [
				"cayman=cayman.__main__:main",
			],
		},
		package_data={},
		include_package_data=True,
		data_files=[],
	)
